@@ -145,8 +145,6 @@ void CWaitCommandsAI::DrawCommands() const
 
 void CWaitCommandsAI::AddTimeWait(const Command& cmd)
 {
-	GML_RECMUTEX_LOCK(sel); // AddTimeWait
-
 	// save the current selection
 	const CUnitSet tmpSet = selectedUnitsHandler.selectedUnits;
 	CUnitSet::const_iterator it;
@@ -440,8 +438,6 @@ bool CWaitCommandsAI::Wait::IsWaitingOn(const CUnit* unit) const
 void CWaitCommandsAI::Wait::SendCommand(const Command& cmd,
 																				const CUnitSet& unitSet)
 {
-	GML_RECMUTEX_LOCK(sel); // SendCommand
-
 	if (unitSet.empty()) {
 		return;
 	}
@@ -668,8 +664,6 @@ CWaitCommandsAI::DeathWait*
 CWaitCommandsAI::DeathWait::DeathWait(const Command& cmd)
 : Wait(CMD_WAITCODE_DEATHWAIT)
 {
-	GML_RECMUTEX_LOCK(sel); // DeathWait
-
 	const CUnitSet& selUnits = selectedUnitsHandler.selectedUnits;
 
 	if (cmd.params.size() == 1) {
@@ -881,8 +875,6 @@ CWaitCommandsAI::SquadWait*
 CWaitCommandsAI::SquadWait::SquadWait(const Command& cmd)
 : Wait(CMD_WAITCODE_SQUADWAIT)
 {
-	GML_RECMUTEX_LOCK(sel); // SquadWait
-
 	if (cmd.params.size() != 1) {
 		return;
 	}
@@ -1037,8 +1029,6 @@ CWaitCommandsAI::GatherWait*
 CWaitCommandsAI::GatherWait::GatherWait(const Command& cmd)
 : Wait(CMD_WAITCODE_GATHERWAIT)
 {
-	GML_RECMUTEX_LOCK(sel); // GatherWait
-
 	if (!cmd.params.empty()) {
 		return;
 	}
