@@ -40,6 +40,7 @@
 #include "Sim/Features/FeatureHandler.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Misc/QuadField.h"
+#include "Sim/Projectiles/Projectile.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitTypes/TransportUnit.h"
@@ -54,6 +55,7 @@
 #include "System/LoadSave/demofile.h"
 #include "System/LoadSave/DemoReader.h"
 #include "System/Log/DefaultFilter.h"
+#include "System/Platform/SDL1_keysym.h"
 #include "System/Sound/SoundChannels.h"
 #include "System/Misc/SpringTime.h"
 
@@ -66,7 +68,8 @@
 #include <list>
 #include <cctype>
 
-#include <SDL_keysym.h>
+#include <SDL_clipboard.h>
+#include <SDL_keycode.h>
 #include <SDL_mouse.h>
 
 const int CMD_INDEX_OFFSET = 1; // starting index for command descriptions
@@ -81,6 +84,11 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	lua_pushcfunction(L, x);    \
 	lua_rawset(L, -3)
 
+#define REGISTER_NAMED_LUA_CFUNC(x,name) \
+	lua_pushstring(L, #name);      \
+	lua_pushcfunction(L, x);    \
+	lua_rawset(L, -3)
+
 	REGISTER_LUA_CFUNC(IsReplay);
 	REGISTER_LUA_CFUNC(GetReplayLength);
 	REGISTER_LUA_CFUNC(GetModUICtrl);
@@ -96,6 +104,7 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetMiniMapGeometry);
 	REGISTER_LUA_CFUNC(GetMiniMapDualScreen);
 	REGISTER_LUA_CFUNC(IsAboveMiniMap);
+	REGISTER_LUA_CFUNC(IsGUIHidden);
 
 	REGISTER_LUA_CFUNC(IsAABBInView);
 	REGISTER_LUA_CFUNC(IsSphereInView);
@@ -125,6 +134,11 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetLocalPlayerID);
 	REGISTER_LUA_CFUNC(GetLocalTeamID);
 	REGISTER_LUA_CFUNC(GetLocalAllyTeamID);
+
+	REGISTER_NAMED_LUA_CFUNC(GetLocalPlayerID,   GetMyPlayerID);
+	REGISTER_NAMED_LUA_CFUNC(GetLocalTeamID,     GetMyTeamID);
+	REGISTER_NAMED_LUA_CFUNC(GetLocalAllyTeamID, GetMyAllyTeamID);
+
 	REGISTER_LUA_CFUNC(GetSpectatingState);
 
 	REGISTER_LUA_CFUNC(GetSelectedUnits);
@@ -132,7 +146,6 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetSelectedUnitsCounts);
 	REGISTER_LUA_CFUNC(GetSelectedUnitsCount);
 
-	REGISTER_LUA_CFUNC(IsGUIHidden);
 	REGISTER_LUA_CFUNC(HaveShadows);
 	REGISTER_LUA_CFUNC(HaveAdvShading);
 	REGISTER_LUA_CFUNC(GetWaterMode);
@@ -155,21 +168,17 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetSoundStreamTime);
 	REGISTER_LUA_CFUNC(GetSoundEffectParams);
 
-	// moved from LuaUI
-
 	REGISTER_LUA_CFUNC(GetFPS);
+	REGISTER_LUA_CFUNC(GetGameSpeed);
 
 	REGISTER_LUA_CFUNC(GetActiveCommand);
 	REGISTER_LUA_CFUNC(GetDefaultCommand);
 	REGISTER_LUA_CFUNC(GetActiveCmdDescs);
 	REGISTER_LUA_CFUNC(GetActiveCmdDesc);
 	REGISTER_LUA_CFUNC(GetCmdDescIndex);
-
 	REGISTER_LUA_CFUNC(GetBuildFacing);
 	REGISTER_LUA_CFUNC(GetBuildSpacing);
-
 	REGISTER_LUA_CFUNC(GetGatherMode);
-
 	REGISTER_LUA_CFUNC(GetActivePage);
 
 	REGISTER_LUA_CFUNC(GetMouseState);
@@ -181,25 +190,20 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetPressedKeys);
 	REGISTER_LUA_CFUNC(GetInvertQueueKey);
 
+	REGISTER_LUA_CFUNC(GetClipboard);
+
 	REGISTER_LUA_CFUNC(GetKeyCode);
 	REGISTER_LUA_CFUNC(GetKeySymbol);
 	REGISTER_LUA_CFUNC(GetKeyBindings);
 	REGISTER_LUA_CFUNC(GetActionHotKeys);
 
 	REGISTER_LUA_CFUNC(GetLastMessagePositions);
-
 	REGISTER_LUA_CFUNC(GetConsoleBuffer);
 	REGISTER_LUA_CFUNC(GetCurrentTooltip);
 
-	REGISTER_LUA_CFUNC(GetMyAllyTeamID);
-	REGISTER_LUA_CFUNC(GetMyTeamID);
-	REGISTER_LUA_CFUNC(GetMyPlayerID);
-
+	REGISTER_LUA_CFUNC(GetUnitGroup);
 	REGISTER_LUA_CFUNC(GetGroupList);
 	REGISTER_LUA_CFUNC(GetSelectedGroup);
-
-	REGISTER_LUA_CFUNC(GetUnitGroup);
-
 	REGISTER_LUA_CFUNC(GetGroupUnits);
 	REGISTER_LUA_CFUNC(GetGroupUnitsSorted);
 	REGISTER_LUA_CFUNC(GetGroupUnitsCounts);
@@ -212,7 +216,6 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetDrawSelectionInfo);
 
 	REGISTER_LUA_CFUNC(GetConfigParams);
-
 	REGISTER_LUA_CFUNC(GetLogSections);
 
 	return true;
@@ -270,15 +273,6 @@ static inline CFeature* ParseFeature(lua_State* L, const char* caller, int index
 
 
 
-static inline void CheckNoArgs(lua_State* L, const char* funcName)
-{
-	const int args = lua_gettop(L); // number of arguments
-	if (args != 0) {
-		luaL_error(L, "%s() takes no arguments", funcName);
-	}
-}
-
-
 /******************************************************************************/
 /******************************************************************************/
 //
@@ -287,7 +281,6 @@ static inline void CheckNoArgs(lua_State* L, const char* funcName)
 
 int LuaUnsyncedRead::IsReplay(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (gameSetup && gameSetup->hostDemo) {
 		lua_pushboolean(L, true);
 	} else {
@@ -299,7 +292,6 @@ int LuaUnsyncedRead::IsReplay(lua_State* L)
 
 int LuaUnsyncedRead::GetReplayLength(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (gameServer && gameServer->GetDemoReader()) {
 		lua_pushnumber(L, gameServer->GetDemoReader()->GetFileHeader().gameTime);
 		return 1;
@@ -310,7 +302,6 @@ int LuaUnsyncedRead::GetReplayLength(lua_State* L)
 
 int LuaUnsyncedRead::GetModUICtrl(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushboolean(L, CLuaHandle::GetModUICtrl());
 	return 1;
 }
@@ -320,7 +311,6 @@ int LuaUnsyncedRead::GetModUICtrl(lua_State* L)
 
 int LuaUnsyncedRead::GetViewGeometry(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, globalRendering->viewSizeX);
 	lua_pushnumber(L, globalRendering->viewSizeY);
 	lua_pushnumber(L, globalRendering->viewPosX);
@@ -331,8 +321,6 @@ int LuaUnsyncedRead::GetViewGeometry(lua_State* L)
 
 int LuaUnsyncedRead::GetWindowGeometry(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
-
 	// origin BOTTOMLEFT
 	const int winPosY_bl = globalRendering->screenSizeY - globalRendering->winSizeY - globalRendering->winPosY;
 
@@ -346,7 +334,6 @@ int LuaUnsyncedRead::GetWindowGeometry(lua_State* L)
 
 int LuaUnsyncedRead::GetScreenGeometry(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, globalRendering->screenSizeX);
 	lua_pushnumber(L, globalRendering->screenSizeY);
 	lua_pushnumber(L, 0.0f);
@@ -418,7 +405,6 @@ int LuaUnsyncedRead::IsAboveMiniMap(lua_State* L)
 
 int LuaUnsyncedRead::GetDrawFrame(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, globalRendering->drawFrame & 0xFFFF);
 	lua_pushnumber(L, (globalRendering->drawFrame >> 16) & 0xFFFF);
 	return 2;
@@ -427,7 +413,6 @@ int LuaUnsyncedRead::GetDrawFrame(lua_State* L)
 
 int LuaUnsyncedRead::GetFrameTimeOffset(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, globalRendering->timeOffset);
 	return 1;
 }
@@ -435,14 +420,12 @@ int LuaUnsyncedRead::GetFrameTimeOffset(lua_State* L)
 
 int LuaUnsyncedRead::GetLastUpdateSeconds(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, game->updateDeltaSeconds);
 	return 1;
 }
 
 int LuaUnsyncedRead::GetHasLag(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushboolean(L, (game != NULL)? game->IsLagging(luaL_optfloat(L, 1, 500.0f)) : false);
 	return 1;
 }
@@ -500,11 +483,7 @@ int LuaUnsyncedRead::IsUnitInView(lua_State* L)
 
 static bool UnitIsIcon(const CUnit* unit)
 {
-	const float sqDist = (unit->pos - camera->GetPos()).SqLength();
-	const float iconLength = unitDrawer->iconLength;
-	const float iconDistSqrMult = unit->unitDef->iconType->GetDistanceSqr();
-	const float realIconLength = iconLength * iconDistSqrMult;
-	return (sqDist > realIconLength);
+	return (unitDrawer->DrawAsIcon(unit, (unit->pos - camera->GetPos()).SqLength()));
 }
 
 
@@ -554,8 +533,6 @@ int LuaUnsyncedRead::IsUnitIcon(lua_State* L)
 
 int LuaUnsyncedRead::IsUnitSelected(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(sel); // IsUnitSelected
-
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
 	if (unit == NULL) {
 		return 0;
@@ -789,8 +766,6 @@ int LuaUnsyncedRead::GetVisibleUnits(lua_State* L)
 	unsigned int count = 0;
 
 	{
-		GML_RECMUTEX_LOCK(quad); // GetVisibleUnits
-
 		unitQuadIter.Reset();
 		readMap->GridVisibility(camera, CQuadField::BASE_QUAD_SIZE / SQUARE_SIZE, 1e9, &unitQuadIter, INT_MAX);
 
@@ -917,8 +892,6 @@ int LuaUnsyncedRead::GetVisibleFeatures(lua_State* L)
 	unsigned int count = 0;
 
 	{
-		GML_RECMUTEX_LOCK(quad); // GetVisibleFeatures
-
 		featureQuadIter.Reset();
 		readMap->GridVisibility(camera, CQuadField::BASE_QUAD_SIZE / SQUARE_SIZE, 3000.0f * 2.0f, &featureQuadIter, INT_MAX);
 
@@ -1005,8 +978,6 @@ int LuaUnsyncedRead::GetVisibleProjectiles(lua_State* L)
 	unsigned int count = 0;
 
 	{
-		GML_RECMUTEX_LOCK(quad); // GetVisibleProjectiles
-
 		projQuadIter.Reset();
 		readMap->GridVisibility(camera, CQuadField::BASE_QUAD_SIZE / SQUARE_SIZE, 1e9, &projQuadIter, INT_MAX);
 
@@ -1060,7 +1031,6 @@ int LuaUnsyncedRead::GetVisibleProjectiles(lua_State* L)
 
 int LuaUnsyncedRead::GetLocalPlayerID(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, gu->myPlayerNum);
 	return 1;
 }
@@ -1068,7 +1038,6 @@ int LuaUnsyncedRead::GetLocalPlayerID(lua_State* L)
 
 int LuaUnsyncedRead::GetLocalTeamID(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, gu->myTeam);
 	return 1;
 }
@@ -1076,7 +1045,6 @@ int LuaUnsyncedRead::GetLocalTeamID(lua_State* L)
 
 int LuaUnsyncedRead::GetLocalAllyTeamID(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, gu->myAllyTeam);
 	return 1;
 }
@@ -1084,7 +1052,6 @@ int LuaUnsyncedRead::GetLocalAllyTeamID(lua_State* L)
 
 int LuaUnsyncedRead::GetSpectatingState(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushboolean(L, gu->spectating);
 	lua_pushboolean(L, gu->spectatingFullView);
 	lua_pushboolean(L, gu->spectatingFullSelect);
@@ -1097,10 +1064,6 @@ int LuaUnsyncedRead::GetSpectatingState(lua_State* L)
 
 int LuaUnsyncedRead::GetSelectedUnits(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(sel); // GetSelectedUnits
-
-	CheckNoArgs(L, __FUNCTION__);
-
 	unsigned int count = 0;
 	const CUnitSet& selUnits = selectedUnitsHandler.selectedUnits;
 
@@ -1118,10 +1081,6 @@ int LuaUnsyncedRead::GetSelectedUnits(lua_State* L)
 
 int LuaUnsyncedRead::GetSelectedUnitsSorted(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(sel); // GetSelectedUnitsSorted
-
-	CheckNoArgs(L, __FUNCTION__);
-
 	map<int, vector<CUnit*> > unitDefMap;
 	map<int, vector<CUnit*> >::const_iterator mit;
 
@@ -1161,10 +1120,6 @@ int LuaUnsyncedRead::GetSelectedUnitsSorted(lua_State* L)
 
 int LuaUnsyncedRead::GetSelectedUnitsCounts(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(sel); // GetSelectedUnitsCounts
-
-	CheckNoArgs(L, __FUNCTION__);
-
 	map<int, int> countMap;
 	map<int, int>::const_iterator mit;
 
@@ -1199,7 +1154,6 @@ int LuaUnsyncedRead::GetSelectedUnitsCounts(lua_State* L)
 
 int LuaUnsyncedRead::GetSelectedUnitsCount(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, selectedUnitsHandler.selectedUnits.size());
 	return 1;
 }
@@ -1210,7 +1164,6 @@ int LuaUnsyncedRead::GetSelectedUnitsCount(lua_State* L)
 
 int LuaUnsyncedRead::IsGUIHidden(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (game == NULL) {
 		return 0;
 	}
@@ -1221,7 +1174,6 @@ int LuaUnsyncedRead::IsGUIHidden(lua_State* L)
 
 int LuaUnsyncedRead::HaveShadows(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (shadowHandler == NULL) {
 		return 0;
 	}
@@ -1232,7 +1184,6 @@ int LuaUnsyncedRead::HaveShadows(lua_State* L)
 
 int LuaUnsyncedRead::HaveAdvShading(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (unitDrawer == NULL) {
 		return 0;
 	}
@@ -1243,7 +1194,6 @@ int LuaUnsyncedRead::HaveAdvShading(lua_State* L)
 
 int LuaUnsyncedRead::GetWaterMode(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (water == NULL) {
 		return 0;
 	}
@@ -1259,8 +1209,6 @@ int LuaUnsyncedRead::GetWaterMode(lua_State* L)
 
 int LuaUnsyncedRead::GetMapDrawMode(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
-
 	switch (readMap->GetGroundDrawer()->GetDrawMode()) {
 		case CBaseGroundDrawer::drawNormal:   { HSTR_PUSH(L, "normal"            ); break; }
 		case CBaseGroundDrawer::drawHeight:   { HSTR_PUSH(L, "height"            ); break; }
@@ -1325,8 +1273,6 @@ int LuaUnsyncedRead::GetMapSquareTexture(lua_State* L)
 
 int LuaUnsyncedRead::GetCameraNames(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
-
 	lua_newtable(L);
 	const std::vector<CCameraController*>& cc = camHandler->GetAvailableControllers();
 	for (size_t i = 0; i < cc.size(); ++i) {
@@ -1341,8 +1287,6 @@ int LuaUnsyncedRead::GetCameraNames(lua_State* L)
 
 int LuaUnsyncedRead::GetCameraState(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
-
 	lua_newtable(L);
 
 	lua_pushliteral(L, "name");
@@ -1364,7 +1308,6 @@ int LuaUnsyncedRead::GetCameraState(lua_State* L)
 
 int LuaUnsyncedRead::GetCameraPosition(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, camera->GetPos().x);
 	lua_pushnumber(L, camera->GetPos().y);
 	lua_pushnumber(L, camera->GetPos().z);
@@ -1374,7 +1317,6 @@ int LuaUnsyncedRead::GetCameraPosition(lua_State* L)
 
 int LuaUnsyncedRead::GetCameraDirection(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, camera->forward.x);
 	lua_pushnumber(L, camera->forward.y);
 	lua_pushnumber(L, camera->forward.z);
@@ -1384,7 +1326,6 @@ int LuaUnsyncedRead::GetCameraDirection(lua_State* L)
 
 int LuaUnsyncedRead::GetCameraFOV(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, camera->GetFov());
 	return 1;
 }
@@ -1392,8 +1333,6 @@ int LuaUnsyncedRead::GetCameraFOV(lua_State* L)
 
 int LuaUnsyncedRead::GetCameraVectors(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
-
 #define PACK_CAMERA_VECTOR(n) \
 	HSTR_PUSH(L, #n);           \
 	lua_createtable(L, 3, 0);            \
@@ -1457,7 +1396,7 @@ int LuaUnsyncedRead::TraceScreenRay(lua_State* L)
 					return 2;
 				}
 			}
-			const float posY = ground->GetHeightReal(pos.x, pos.z, false);
+			const float posY = CGround::GetHeightReal(pos.x, pos.z, false);
 			lua_pushliteral(L, "ground");
 			lua_createtable(L, 3, 0);
 			lua_pushnumber(L, pos.x); lua_rawseti(L, -2, 1);
@@ -1762,7 +1701,6 @@ int LuaUnsyncedRead::GetSoundEffectParams(lua_State* L)
 
 int LuaUnsyncedRead::GetFPS(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (globalRendering) {
 		lua_pushnumber(L, (int)globalRendering->FPS);
 	} else {
@@ -1772,16 +1710,22 @@ int LuaUnsyncedRead::GetFPS(lua_State* L)
 }
 
 
+int LuaUnsyncedRead::GetGameSpeed(lua_State* L)
+{
+	lua_pushnumber(L, gs->wantedSpeedFactor);
+	lua_pushnumber(L, gs->speedFactor);
+	lua_pushboolean(L, gs->paused);
+	return 3;
+}
+
+
 /******************************************************************************/
 
 int LuaUnsyncedRead::GetActiveCommand(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(gui); // GetActiveCommand
-
 	if (guihandler == NULL) {
 		return 0;
 	}
-	CheckNoArgs(L, __FUNCTION__);
 
 	const vector<CommandDescription>& cmdDescs = guihandler->commands;
 	const int cmdDescCount = (int)cmdDescs.size();
@@ -1803,12 +1747,9 @@ int LuaUnsyncedRead::GetDefaultCommand(lua_State* L)
 	if (guihandler == NULL) {
 		return 0;
 	}
-	CheckNoArgs(L, __FUNCTION__);
 
 	const int defCmd = guihandler->GetDefaultCommand(mouse->lastx, mouse->lasty);
 
-	GML_RECMUTEX_LOCK(gui); // GetDefaultCommand
-
 	const vector<CommandDescription>& cmdDescs = guihandler->commands;
 	const int cmdDescCount = (int)cmdDescs.size();
 
@@ -1825,12 +1766,9 @@ int LuaUnsyncedRead::GetDefaultCommand(lua_State* L)
 
 int LuaUnsyncedRead::GetActiveCmdDescs(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(gui); // GetActiveCmdDescs
-
 	if (guihandler == NULL) {
 		return 0;
 	}
-	CheckNoArgs(L, __FUNCTION__);
 
 	const vector<CommandDescription>& cmdDescs = guihandler->commands;
 	const int cmdDescCount = (int)cmdDescs.size();
@@ -1848,8 +1786,6 @@ int LuaUnsyncedRead::GetActiveCmdDescs(lua_State* L)
 
 int LuaUnsyncedRead::GetActiveCmdDesc(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(gui); // GetActiveCmdDesc
-
 	if (guihandler == NULL) {
 		return 0;
 	}
@@ -1867,8 +1803,6 @@ int LuaUnsyncedRead::GetActiveCmdDesc(lua_State* L)
 
 int LuaUnsyncedRead::GetCmdDescIndex(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(gui); // GetCmdDescIndex
-
 	if (guihandler == NULL) {
 		return 0;
 	}
@@ -1893,7 +1827,6 @@ int LuaUnsyncedRead::GetBuildFacing(lua_State* L)
 	if (guihandler == NULL) {
 		return 0;
 	}
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, guihandler->buildFacing);
 	return 1;
 }
@@ -1904,7 +1837,6 @@ int LuaUnsyncedRead::GetBuildSpacing(lua_State* L)
 	if (guihandler == NULL) {
 		return 0;
 	}
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, guihandler->buildSpacing);
 	return 1;
 }
@@ -1915,7 +1847,6 @@ int LuaUnsyncedRead::GetGatherMode(lua_State* L)
 	if (guihandler == NULL) {
 		return 0;
 	}
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, guihandler->GetGatherMode());
 	return 1;
 }
@@ -1938,7 +1869,6 @@ int LuaUnsyncedRead::GetActivePage(lua_State* L)
 
 int LuaUnsyncedRead::GetMouseState(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, mouse->lastx - globalRendering->viewPosX);
 	lua_pushnumber(L, globalRendering->viewSizeY - mouse->lasty - 1);
 	lua_pushboolean(L, mouse->buttons[SDL_BUTTON_LEFT].pressed);
@@ -1950,7 +1880,6 @@ int LuaUnsyncedRead::GetMouseState(lua_State* L)
 
 int LuaUnsyncedRead::GetMouseCursor(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushsstring(L, mouse->GetCurrentCursor());
 	lua_pushnumber(L, mouse->GetCurrentCursorScale());
 	return 2;
@@ -1985,38 +1914,28 @@ int LuaUnsyncedRead::GetMouseStartPosition(lua_State* L)
 int LuaUnsyncedRead::GetKeyState(lua_State* L)
 {
 	const int key = luaL_checkint(L, 1);
-	if ((key < 0) || (key >= SDLK_LAST)) {
-		lua_pushboolean(L, 0);
-	} else {
-		lua_pushboolean(L, keyInput->GetKeyState(key));
-	}
+	lua_pushboolean(L, KeyInput::IsKeyPressed(key));
 	return 1;
 }
 
 
 int LuaUnsyncedRead::GetModKeyState(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
-	lua_pushboolean(L, keyInput->GetKeyState(SDLK_LALT));
-	lua_pushboolean(L, keyInput->GetKeyState(SDLK_LCTRL));
-	lua_pushboolean(L, keyInput->GetKeyState(SDLK_LMETA));
-	lua_pushboolean(L, keyInput->GetKeyState(SDLK_LSHIFT));
+	lua_pushboolean(L, KeyInput::GetKeyModState(KMOD_ALT));
+	lua_pushboolean(L, KeyInput::GetKeyModState(KMOD_CTRL));
+	lua_pushboolean(L, KeyInput::GetKeyModState(KMOD_GUI));
+	lua_pushboolean(L, KeyInput::GetKeyModState(KMOD_SHIFT));
 	return 4;
 }
 
 
 int LuaUnsyncedRead::GetPressedKeys(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_newtable(L);
-
-	unsigned int count = 0;
-
-	for (int i = 0; i < SDLK_LAST; i++) {
-		if (keyInput->GetKeyState(i)) {
-			lua_pushboolean(L, 1);
-			lua_rawseti(L, -2, i);
-			count++;
+	for (auto key: KeyInput::GetPressedKeys()) {
+		if (key.second) {
+			lua_pushboolean(L, true);
+			lua_rawseti(L, -2, key.first);
 		}
 	}
 	return 1;
@@ -2025,7 +1944,6 @@ int LuaUnsyncedRead::GetPressedKeys(lua_State* L)
 
 int LuaUnsyncedRead::GetInvertQueueKey(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	if (guihandler == NULL) {
 		return 0;
 	}
@@ -2035,16 +1953,28 @@ int LuaUnsyncedRead::GetInvertQueueKey(lua_State* L)
 
 /******************************************************************************/
 
+int LuaUnsyncedRead::GetClipboard(lua_State* L)
+{
+	char* text = SDL_GetClipboardText();
+	if (text == NULL) {
+		return 0;
+	}
+	lua_pushstring(L, text);
+	SDL_free(text);
+	return 1;
+}
+
+/******************************************************************************/
+
 int LuaUnsyncedRead::GetLastMessagePositions(lua_State* L)
 {
 	CInfoConsole* ic = game->infoConsole;
-	if (ic == NULL) {
-		return 0;
-	}
 
-	CheckNoArgs(L, __FUNCTION__);
+	if (ic == NULL)
+		return 0;
+
 	lua_newtable(L);
-	for (int i=1; i<=ic->GetMsgPosCount(); i++) {
+	for (unsigned int i = 1; i <= ic->GetMsgPosCount(); i++) {
 		lua_newtable(L); {
 			const float3 msgpos = ic->GetMsgPos();
 			lua_pushnumber(L, msgpos.x); lua_rawseti(L, -2, 1);
@@ -2091,10 +2021,8 @@ int LuaUnsyncedRead::GetConsoleBuffer(lua_State* L)
 			lua_pushliteral(L, "text");
 			lua_pushsstring(L, lines[i].text);
 			lua_rawset(L, -3);
-			// FIXME migrate priority to level...
 			lua_pushliteral(L, "priority");
-			lua_pushnumber(L, 0);
-			//lua_pushstring(L, lines[i].level);
+			lua_pushnumber(L, lines[i].level);
 			lua_rawset(L, -3);
 		}
 		lua_rawset(L, -3);
@@ -2106,7 +2034,6 @@ int LuaUnsyncedRead::GetConsoleBuffer(lua_State* L)
 
 int LuaUnsyncedRead::GetCurrentTooltip(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushsstring(L, mouse->GetCurrentTooltip());
 	return 1;
 }
@@ -2114,14 +2041,19 @@ int LuaUnsyncedRead::GetCurrentTooltip(lua_State* L)
 
 int LuaUnsyncedRead::GetKeyCode(lua_State* L)
 {
-	lua_pushnumber(L, (keyCodes != NULL)? keyCodes->GetCode(luaL_checksstring(L, 1)): -1);
+	if (keyCodes == NULL) {
+		return 0;
+	}
+
+	const int keycode = keyCodes->GetCode(luaL_checksstring(L, 1));
+	lua_pushnumber(L, SDL21_keysyms(keycode));
 	return 1;
 }
 
 
 int LuaUnsyncedRead::GetKeySymbol(lua_State* L)
 {
-	const int keycode = luaL_checkint(L, 1);
+	const int keycode = SDL12_keysyms(luaL_checkint(L, 1));
 	lua_pushsstring(L, (keyCodes != NULL)? keyCodes->GetName(keycode): "");
 	lua_pushsstring(L, (keyCodes != NULL)? keyCodes->GetDefaultName(keycode): "");
 	return 2;
@@ -2139,14 +2071,17 @@ int LuaUnsyncedRead::GetKeyBindings(lua_State* L)
 
 	const CKeyBindings::ActionList& actions = keyBindings->GetActionList(ks);
 
+	int i = 1;
 	lua_newtable(L);
-	for (unsigned int i = 0; i < actions.size(); i++) {
-		const Action& action = actions[i];
+	for (const Action& action: actions) {
 		lua_newtable(L);
-		lua_pushsstring(L, action.command);
-		lua_pushsstring(L, action.extra);
-		lua_rawset(L, -3);
-		lua_rawseti(L, -2, i + 1);
+			lua_pushsstring(L, action.command);
+			lua_pushsstring(L, action.extra);
+			lua_rawset(L, -3);
+			LuaPushNamedString(L, "command",   action.command);
+			LuaPushNamedString(L, "extra",     action.extra);
+			LuaPushNamedString(L, "boundWith", action.boundWith);
+		lua_rawseti(L, -2, i++);
 	}
 	return 1;
 }
@@ -2160,10 +2095,10 @@ int LuaUnsyncedRead::GetActionHotKeys(lua_State* L)
 	const CKeyBindings::HotkeyList& hotkeys = keyBindings->GetHotkeys(luaL_checksstring(L, 1));
 
 	lua_newtable(L);
-	for (unsigned int i = 0; i < hotkeys.size(); i++) {
-		const string& hotkey = hotkeys[i];
+	int i = 1;
+	for (const std::string& hotkey: hotkeys) {
 		lua_pushsstring(L, hotkey);
-		lua_rawseti(L, -2, i + 1);
+		lua_rawseti(L, -2, i++);
 	}
 	return 1;
 }
@@ -2172,9 +2107,6 @@ int LuaUnsyncedRead::GetActionHotKeys(lua_State* L)
 
 int LuaUnsyncedRead::GetGroupList(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // GetGroupList
-
-	CheckNoArgs(L, __FUNCTION__);
 	if (grouphandlers[gu->myTeam] == NULL) {
 		return 0;
 	}
@@ -2199,7 +2131,6 @@ int LuaUnsyncedRead::GetGroupList(lua_State* L)
 
 int LuaUnsyncedRead::GetSelectedGroup(lua_State* L)
 {
-	CheckNoArgs(L, __FUNCTION__);
 	lua_pushnumber(L, selectedUnitsHandler.GetSelectedGroup());
 	return 1;
 }
@@ -2223,8 +2154,6 @@ int LuaUnsyncedRead::GetUnitGroup(lua_State* L)
 
 int LuaUnsyncedRead::GetGroupUnits(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // GetGroupUnits
-
 	const int groupID = luaL_checkint(L, 1);
 	const vector<CGroup*>& groups = grouphandlers[gu->myTeam]->groups;
 	if ((groupID < 0) || ((size_t)groupID >= groups.size()) ||
@@ -2250,8 +2179,6 @@ int LuaUnsyncedRead::GetGroupUnits(lua_State* L)
 
 int LuaUnsyncedRead::GetGroupUnitsSorted(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // GetGroupUnitsSorted
-
 	const int groupID = luaL_checkint(L, 1);
 	const vector<CGroup*>& groups = grouphandlers[gu->myTeam]->groups;
 	if ((groupID < 0) || ((size_t)groupID >= groups.size()) ||
@@ -2288,8 +2215,6 @@ int LuaUnsyncedRead::GetGroupUnitsSorted(lua_State* L)
 
 int LuaUnsyncedRead::GetGroupUnitsCounts(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // GetGroupUnitsCounts
-
 	const int groupID = luaL_checkint(L, 1);
 	const vector<CGroup*>& groups = grouphandlers[gu->myTeam]->groups;
 	if ((groupID < 0) || ((size_t)groupID >= groups.size()) ||
@@ -2324,8 +2249,6 @@ int LuaUnsyncedRead::GetGroupUnitsCounts(lua_State* L)
 
 int LuaUnsyncedRead::GetGroupUnitsCount(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // GetGroupUnitsCount
-
 	const int groupID = luaL_checkint(L, 1);
 	const vector<CGroup*>& groups = grouphandlers[gu->myTeam]->groups;
 	if ((groupID < 0) || ((size_t)groupID >= groups.size()) ||
@@ -2337,33 +2260,6 @@ int LuaUnsyncedRead::GetGroupUnitsCount(lua_State* L)
 }
 
 
-/******************************************************************************/
-/******************************************************************************/
-
-int LuaUnsyncedRead::GetMyPlayerID(lua_State* L)
-{
-	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->myPlayerNum);
-	return 1;
-}
-
-
-int LuaUnsyncedRead::GetMyTeamID(lua_State* L)
-{
-	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->myTeam);
-	return 1;
-}
-
-
-int LuaUnsyncedRead::GetMyAllyTeamID(lua_State* L)
-{
-	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->myAllyTeam);
-	return 1;
-}
-
-
 /******************************************************************************/
 /******************************************************************************/
 
@@ -2406,17 +2302,19 @@ int LuaUnsyncedRead::GetPlayerTraffic(lua_State* L)
 	}
 
 	// only allow viewing stats for specific packet types
-	if ((playerID != -1) &&              // all system counts can be read
-	    (playerID != gu->myPlayerNum) && // all  self  counts can be read
-	    (packetID != -1) &&
-	    (packetID != NETMSG_CHAT)     &&
-	    (packetID != NETMSG_PAUSE)    &&
-	    (packetID != NETMSG_LUAMSG)   &&
-	    (packetID != NETMSG_STARTPOS) &&
-	    (packetID != NETMSG_USER_SPEED)) {
-    lua_pushnumber(L, -1);
-    return 1;
-  }
+	if (
+		(playerID != -1) &&              // all system counts can be read
+		(playerID != gu->myPlayerNum) && // all  self  counts can be read
+		(packetID != -1) &&
+		(packetID != NETMSG_CHAT)     &&
+		(packetID != NETMSG_PAUSE)    &&
+		(packetID != NETMSG_LUAMSG)   &&
+		(packetID != NETMSG_STARTPOS) &&
+		(packetID != NETMSG_USER_SPEED)
+	) {
+		lua_pushnumber(L, -1);
+		return 1;
+	}
 
 	const CGame::PlayerTrafficInfo& pti = it->second;
 	if (packetID == -1) {
