@@ -41,7 +41,6 @@ class Action;
 class CDemoRecorder;
 class AutohostInterface;
 class CGameSetup;
-class ClientSetup;
 class ChatMessage;
 class GameParticipant;
 class GameSkirmishAI;
@@ -72,6 +71,8 @@ public:
 	CGameServer(const std::string& hostIP, int hostPort, const GameData* const gameData, const CGameSetup* const setup);
 	~CGameServer();
 
+	CGameServer(const CGameServer&) = delete; // no-copy
+
 	void AddLocalClient(const std::string& myName, const std::string& myVersion);
 
 	void AddAutohostInterface(const std::string& autohostIP, const int autohostPort);
