@@ -20,7 +20,7 @@
 #include "Game/Players/PlayerHandler.h"
 #include "Game/UI/GameSetupDrawer.h"
 #include "Game/UI/MouseHandler.h"
-#include "Lua/LuaRules.h"
+#include "Lua/LuaHandle.h"
 #include "Rendering/GlobalRendering.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Path/IPathManager.h"
@@ -40,6 +40,9 @@
 LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_NET)
 
 
+static std::map<int, unsigned int> mySyncChecksums;
+
+
 void CGame::AddTraffic(int playerID, int packetCode, int length)
 {
 	std::map<int, PlayerTrafficInfo>::iterator it = playerTraffic.find(playerID);
@@ -68,7 +71,7 @@ void CGame::SendClientProcUsage()
 		if (playing) {
 			const float simProcUsage = (profiler.GetPercent("SimFrame"));
 			const float drawProcUsage = (profiler.GetPercent("GameController::Draw") / std::max(1.0f, globalRendering->FPS)) * gu->minFPS;
-			const float totalProcUsage = simProcUsage + drawProcUsage * (!GML::SimEnabled() || !GML::MultiThreadSim());
+			const float totalProcUsage = simProcUsage + drawProcUsage;
 
 			// take the minimum drawframes into account, too
 			net->Send(CBaseNetProtocol::Get().SendCPUUsage(totalProcUsage));
@@ -204,15 +207,7 @@ float CGame::GetNetMessageProcessingTimeLimit() const
 	const float minDrawFPS   =         gu->reconnectSimDrawBalance  * 1000.0f / std::max(0.01f, gu->avgDrawFrameTime);
 	const float simDrawRatio = maxSimFPS / minDrawFPS;
 
-	float limit = 0.0f;
-
-	if (GML::SimEnabled() && GML::MultiThreadSim()) {
-		limit = (1000.0f / gu->minFPS);
-	} else {
-		limit = Clamp(simDrawRatio * gu->avgSimFrameTime, 5.0f, 1000.0f / gu->minFPS);
-	}
-
-	return limit;
+	return Clamp(simDrawRatio * gu->avgSimFrameTime, 5.0f, 1000.0f / gu->minFPS);
 }
 
 void CGame::ClientReadNet()
@@ -335,8 +330,6 @@ void CGame::ClientReadNet()
 			}
 
 			case NETMSG_USER_SPEED: {
-				gs->wantedSpeedFactor = *((float*) &inbuf[2]);
-
 				const unsigned char player = inbuf[1];
 				if (!playerHandler->IsValidPlayer(player) && player != SERVER_PLAYER) {
 					LOG_L(L_ERROR, "Got invalid player num %i in user speed msg", player);
@@ -344,6 +337,7 @@ void CGame::ClientReadNet()
 				}
 				const char* pName = (player == SERVER_PLAYER)? "server": playerHandler->Player(player)->name.c_str();
 
+				gs->wantedSpeedFactor = *((float*) &inbuf[2]);
 				LOG("Speed set to %.1f [%s]", gs->wantedSpeedFactor, pName);
 				AddTraffic(player, packetCode, dataLength);
 				break;
@@ -432,7 +426,7 @@ void CGame::ClientReadNet()
 					CTeam* team = teamHandler->Team(teamID);
 					team->ClampStartPosInStartBox(&clampedPos);
 
-					if (!luaRules || luaRules->AllowStartPosition(playerID, rdyState, clampedPos, rawPickPos)) {
+					if (eventHandler.AllowStartPosition(playerID, rdyState, clampedPos, rawPickPos)) {
 						team->SetStartPos(clampedPos);
 
 						if (playerID != SERVER_PLAYER) {
@@ -516,6 +510,11 @@ void CGame::ClientReadNet()
 				ASSERT_SYNCED(CSyncChecker::GetChecksum());
 				net->Send(CBaseNetProtocol::Get().SendSyncResponse(gu->myPlayerNum, gs->frameNum, CSyncChecker::GetChecksum()));
 
+				if (gameServer != NULL && gameServer->GetDemoReader() != NULL) {
+					// buffer all checksums, so we can check sync later between demo & local
+					mySyncChecksums[gs->frameNum] = CSyncChecker::GetChecksum();
+				}
+
 				if ((gs->frameNum & 4095) == 0) {
 					// reset checksum every 4096 frames =~ 2.5 minutes
 					CSyncChecker::NewFrame();
@@ -538,10 +537,11 @@ void CGame::ClientReadNet()
 					netcode::UnpackPacket pckt(packet, 1);
 
 					unsigned char playerNum; pckt >> playerNum;
-						      int  frameNum; pckt >> frameNum;
+						  int  frameNum; pckt >> frameNum;
 					unsigned  int  checkSum; pckt >> checkSum;
 
-					const unsigned int ourCheckSum = CSyncChecker::GetChecksum();
+					const unsigned int ourCheckSum = mySyncChecksums[frameNum];
+
 					const char* fmtStr =
 						"[DESYNC_WARNING] checksum %x from player %d (%s)"
 						" does not match our checksum %x for frame-number %d";
@@ -550,11 +550,10 @@ void CGame::ClientReadNet()
 					// check if our checksum for this frame matches what
 					// player <playerNum> sent to the server at the same
 					// frame in the original game (in case of a demo)
-					if (playerNum == gu->myPlayerNum) { return; }
-					if (gs->frameNum != frameNum) { return; }
-					if (checkSum == ourCheckSum) { return; }
+					if (playerNum == gu->myPlayerNum) { break; }
+					if (checkSum == ourCheckSum) { break; }
 
-					LOG_L(L_ERROR, fmtStr, checkSum, playerNum, player->name.c_str(), ourCheckSum, gs->frameNum);
+					LOG_L(L_ERROR, fmtStr, checkSum, playerNum, player->name.c_str(), ourCheckSum, frameNum);
 				}
 #endif
 			} break;
@@ -780,13 +779,13 @@ void CGame::ClientReadNet()
 					pckt >> energyShare;
 
 					if (metalShare > 0.0f) {
-						if (!luaRules || luaRules->AllowResourceTransfer(srcTeam, dstTeam, "m", metalShare)) {
+						if (eventHandler.AllowResourceTransfer(srcTeam, dstTeam, "m", metalShare)) {
 							teamHandler->Team(srcTeam)->metal -= metalShare;
 							teamHandler->Team(dstTeam)->metal += metalShare;
 						}
 					}
 					if (energyShare > 0.0f) {
-						if (!luaRules || luaRules->AllowResourceTransfer(srcTeam, dstTeam, "e", energyShare)) {
+						if (eventHandler.AllowResourceTransfer(srcTeam, dstTeam, "e", energyShare)) {
 							teamHandler->Team(srcTeam)->energy -= energyShare;
 							teamHandler->Team(dstTeam)->energy += energyShare;
 						}
@@ -851,7 +850,7 @@ void CGame::ClientReadNet()
 				const float energyShare = Clamp(*(float*)&inbuf[8], 0.0f, (float)srcTeam->energy);
 
 				if (metalShare > 0.0f) {
-					if (!luaRules || luaRules->AllowResourceTransfer(srcTeamID, dstTeamID, "m", metalShare)) {
+					if (eventHandler.AllowResourceTransfer(srcTeamID, dstTeamID, "m", metalShare)) {
 						srcTeam->metal                       -= metalShare;
 						srcTeam->metalSent                   += metalShare;
 						srcTeam->currentStats->metalSent     += metalShare;
@@ -861,7 +860,7 @@ void CGame::ClientReadNet()
 					}
 				}
 				if (energyShare > 0.0f) {
-					if (!luaRules || luaRules->AllowResourceTransfer(srcTeamID, dstTeamID, "e", energyShare)) {
+					if (eventHandler.AllowResourceTransfer(srcTeamID, dstTeamID, "e", energyShare)) {
 						srcTeam->energy                       -= energyShare;
 						srcTeam->energySent                   += energyShare;
 						srcTeam->currentStats->energySent     += energyShare;
@@ -916,10 +915,10 @@ void CGame::ClientReadNet()
 				float metalShare=*(float*)&inbuf[3];
 				float energyShare=*(float*)&inbuf[7];
 
-				if (!luaRules || luaRules->AllowResourceLevel(team, "m", metalShare)) {
+				if (eventHandler.AllowResourceLevel(team, "m", metalShare)) {
 					teamHandler->Team(team)->metalShare = metalShare;
 				}
-				if (!luaRules || luaRules->AllowResourceLevel(team, "e", energyShare)) {
+				if (eventHandler.AllowResourceLevel(team, "e", energyShare)) {
 					teamHandler->Team(team)->energyShare = energyShare;
 				}
 				AddTraffic(player, packetCode, dataLength);
