@@ -26,7 +26,7 @@
 #include "Sim/Path/Default/PathFlowMap.hpp"
 #undef private
 
-#include "Rendering/glFont.h"
+#include "Rendering/Fonts/glFont.h"
 #include "Rendering/DefaultPathDrawer.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/glExtra.h"
@@ -56,7 +56,7 @@ DefaultPathDrawer::DefaultPathDrawer(): IPathDrawer()
 
 void DefaultPathDrawer::DrawAll() const {
 	// CPathManager is not thread-safe
-	if (!GML::SimEnabled() && enabled && (gs->cheatEnabled || gu->spectating)) {
+	if (enabled && (gs->cheatEnabled || gu->spectating)) {
 		glPushAttrib(GL_ENABLE_BIT);
 
 		Draw();
@@ -72,7 +72,7 @@ void DefaultPathDrawer::DrawAll() const {
 void DefaultPathDrawer::DrawInMiniMap()
 {
 	const CBaseGroundDrawer* gd = readMap->GetGroundDrawer();
-	const auto pe = pm->medResPE;
+	const CPathEstimator* pe = pm->medResPE;
 	const MoveDef* md = GetSelectedMoveDef();
 
 	if (md == NULL)
@@ -146,8 +146,6 @@ void DefaultPathDrawer::UpdateExtraTexture(int extraTex, int starty, int endy, i
 
 							CFeature* f = NULL;
 
-							GML_RECMUTEX_LOCK(quad); // UpdateExtraTexture - testunitbuildsquare accesses features in the quadfield
-
 							if (CGameHelper::TestUnitBuildSquare(bi, f, gu->myAllyTeam, false)) {
 								if (f != NULL) {
 									status = OBJECTBLOCKED;
@@ -364,7 +362,7 @@ void DefaultPathDrawer::Draw(const CPathFinder* pf) const {
 		float3 p1;
 			p1.x = sqr.x * SQUARE_SIZE;
 			p1.z = sqr.y * SQUARE_SIZE;
-			p1.y = ground->GetHeightAboveWater(p1.x, p1.z, false) + 15.0f;
+			p1.y = CGround::GetHeightAboveWater(p1.x, p1.z, false) + 15.0f;
 		float3 p2;
 
 		if (!camera->InView(p1) && !camera->InView(p2))
@@ -381,7 +379,7 @@ void DefaultPathDrawer::Draw(const CPathFinder* pf) const {
 		*/
 			p2.x = obx * SQUARE_SIZE;
 			p2.z = obz * SQUARE_SIZE;
-			p2.y = ground->GetHeightAboveWater(p2.x, p2.z, false) + 15.0f;
+			p2.y = CGround::GetHeightAboveWater(p2.x, p2.z, false) + 15.0f;
 
 			glVertexf3(p1);
 			glVertexf3(p2);
@@ -394,8 +392,6 @@ void DefaultPathDrawer::Draw(const CPathFinder* pf) const {
 
 
 void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
-	GML_RECMUTEX_LOCK(sel); // Draw
-
 	const MoveDef* md = GetSelectedMoveDef();
 	const PathNodeStateBuffer& blockStates = pe->blockStates;
 
@@ -427,7 +423,7 @@ void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
 				float3 p1;
 					p1.x = (blockStates.peNodeOffsets[blockNr][md->pathType].x) * SQUARE_SIZE;
 					p1.z = (blockStates.peNodeOffsets[blockNr][md->pathType].y) * SQUARE_SIZE;
-					p1.y = ground->GetHeightAboveWater(p1.x, p1.z, false) + 10.0f;
+					p1.y = CGround::GetHeightAboveWater(p1.x, p1.z, false) + 10.0f;
 
 				if (!camera->InView(p1))
 					continue;
@@ -454,7 +450,7 @@ void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
 					float3 p2;
 						p2.x = (blockStates.peNodeOffsets[obBlockNr][md->pathType].x) * SQUARE_SIZE;
 						p2.z = (blockStates.peNodeOffsets[obBlockNr][md->pathType].y) * SQUARE_SIZE;
-						p2.y = ground->GetHeightAboveWater(p2.x, p2.z, false) + 10.0f;
+						p2.y = CGround::GetHeightAboveWater(p2.x, p2.z, false) + 10.0f;
 
 					glColor3f(1.0f / math::sqrtf(cost), 1.0f / cost, peBlueValue);
 					glVertexf3(p1);
@@ -472,7 +468,7 @@ void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
 				float3 p1;
 					p1.x = (blockStates.peNodeOffsets[blockNr][md->pathType].x) * SQUARE_SIZE;
 					p1.z = (blockStates.peNodeOffsets[blockNr][md->pathType].y) * SQUARE_SIZE;
-					p1.y = ground->GetHeightAboveWater(p1.x, p1.z, false) + 10.0f;
+					p1.y = CGround::GetHeightAboveWater(p1.x, p1.z, false) + 10.0f;
 
 				if (!camera->InView(p1))
 					continue;
@@ -495,7 +491,7 @@ void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
 					float3 p2;
 						p2.x = (blockStates.peNodeOffsets[obBlockNr][md->pathType].x) * SQUARE_SIZE;
 						p2.z = (blockStates.peNodeOffsets[obBlockNr][md->pathType].y) * SQUARE_SIZE;
-						p2.y = ground->GetHeightAboveWater(p2.x, p2.z, false) + 10.0f;
+						p2.y = CGround::GetHeightAboveWater(p2.x, p2.z, false) + 10.0f;
 
 					p2 = (p1 + p2) / 2.0f;
 
@@ -535,11 +531,11 @@ void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
 			float3 p1;
 				p1.x = (blockStates.peNodeOffsets[blockNr][md->pathType].x) * SQUARE_SIZE;
 				p1.z = (blockStates.peNodeOffsets[blockNr][md->pathType].y) * SQUARE_SIZE;
-				p1.y = ground->GetHeightAboveWater(p1.x, p1.z, false) + 15.0f;
+				p1.y = CGround::GetHeightAboveWater(p1.x, p1.z, false) + 15.0f;
 			float3 p2;
 				p2.x = (blockStates.peNodeOffsets[obBlockNr][md->pathType].x) * SQUARE_SIZE;
 				p2.z = (blockStates.peNodeOffsets[obBlockNr][md->pathType].y) * SQUARE_SIZE;
-				p2.y = ground->GetHeightAboveWater(p2.x, p2.z, false) + 15.0f;
+				p2.y = CGround::GetHeightAboveWater(p2.x, p2.z, false) + 15.0f;
 
 			if (!camera->InView(p1) && !camera->InView(p2))
 				continue;
@@ -562,7 +558,7 @@ void DefaultPathDrawer::Draw(const CPathEstimator* pe) const {
 			float3 p1;
 				p1.x = (blockStates.peNodeOffsets[blockNr][md->pathType].x) * SQUARE_SIZE;
 				p1.z = (blockStates.peNodeOffsets[blockNr][md->pathType].y) * SQUARE_SIZE;
-				p1.y = ground->GetHeightAboveWater(p1.x, p1.z, false) + 35.0f;
+				p1.y = CGround::GetHeightAboveWater(p1.x, p1.z, false) + 35.0f;
 
 			if (!camera->InView(p1))
 				continue;
