@@ -1112,8 +1112,6 @@ void CDynWater::AddShipWakes()
 	va2->Initialize();
 
 	{
-		GML_RECMUTEX_LOCK(unit); // AddShipWakes
-
 		const std::set<CUnit*>& units = unitDrawer->GetUnsortedUnits();
 		const int nadd = units.size() * 4;
 
@@ -1203,8 +1201,6 @@ void CDynWater::AddShipWakes()
 
 void CDynWater::AddExplosions()
 {
-	GML_STDMUTEX_LOCK(water); // AddExplosions
-
 	if (explosions.empty()) {
 		return;
 	}
@@ -1294,8 +1290,6 @@ void CDynWater::AddExplosion(const float3& pos, float strength, float size)
 		return;
 	}
 
-	GML_STDMUTEX_LOCK(water); // AddExplosion
-
 	explosions.push_back(Explosion(pos, std::min(size*20, strength), size));
 }
 
