@@ -221,7 +221,7 @@ bool CCollisionHandler::IntersectPieceTreeHelper(
 	const CMatrix44f& mat,
 	const float3& p0,
 	const float3& p1,
-	std::list<CollisionQuery>* cqs
+	std::vector<CollisionQuery>* cqs
 ) {
 	bool ret = false;
 
@@ -252,12 +252,14 @@ bool CCollisionHandler::IntersectPiecesHelper(
 	const CUnit* u,
 	const float3& p0,
 	const float3& p1,
-	std::list<CollisionQuery>* cqs
+	std::vector<CollisionQuery>* cqs
 ) {
 	CMatrix44f unitMat = u->GetTransformMatrix(true);
 	CMatrix44f volMat;
 	CollisionQuery cq;
 
+	cqs->reserve(u->localModel->pieces.size());
+
 	for (unsigned int n = 0; n < u->localModel->pieces.size(); n++) {
 		const LocalModelPiece* lmp = u->localModel->GetPiece(n);
 		const CollisionVolume* lmpVol = lmp->GetCollisionVolume();
@@ -285,8 +287,8 @@ bool CCollisionHandler::IntersectPiecesHelper(
 
 bool CCollisionHandler::IntersectPieceTree(const CUnit* u, const float3& p0, const float3& p1, CollisionQuery* cq)
 {
-	std::list<CollisionQuery> cqs;
-	std::list<CollisionQuery>::const_iterator cqsIt;
+	std::vector<CollisionQuery> cqs;
+	std::vector<CollisionQuery>::const_iterator cqsIt;
 
 	// TODO:
 	//   needs an early-out test, but gets complicated because
@@ -301,11 +303,13 @@ bool CCollisionHandler::IntersectPieceTree(const CUnit* u, const float3& p0, con
 		return true;
 
 	float minDstSq = std::numeric_limits<float>::max();
-	float curDstSq = 0.0f;
 
 	// save the closest intersection
+	// TODO: merge this with helper?
 	for (cqsIt = cqs.begin(); cqsIt != cqs.end(); ++cqsIt) {
-		if ((curDstSq = (cqsIt->GetHitPos() - p0).SqLength()) >= minDstSq)
+		const float curDstSq = (cqsIt->GetHitPos() - p0).SqLength();
+
+		if (curDstSq >= minDstSq)
 			continue;
 
 		minDstSq = curDstSq;
